// Pocket Kloostermania
// Copyright E. Kowalski 2010
// License: GPLv2, see file COPYING in the source tar file

package net.frogparrot.android.kloostermania;

import java.util.Random;

//import net.frogparrot.android.*;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.Html;
import android.util.Log; // Might be useful
import android.view.GestureDetector;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
//import android.view.View.OnTouchListener;
import android.widget.Toast;

public class Kloostermania extends Activity {
	
	KloosView kv; // Main view
	// Version number
	private static final String version = "0.3";
	// A random number generator
	private static Random rng;
	// True if only prime moduli are shown
	boolean onlyPrimes = true;
	// True is slideshow is playing
	boolean slideShow = false;
	
	// Constants for the options meanu
	private static final int MENU_NEW_KLOOS = 0;
	private static final int MENU_ABOUT = 1;
	private static final int MENU_MOD = 2;
	//private static final int MENU_NEXT = 3;
	private static final int MENU_RANDOM = 4;
	private static final int MENU_PRIMES = 5;
	private static final int MENU_SLIDE = 6;
	
	// Constants for the type of sums shown
	public final static int KLOOS_TYPE_KLOOS = 0;
	public final static int KLOOS_TYPE_SALIE = 1;
	public final static int KLOOS_TYPE_BOTH = 2;
	// Default is only Kloosterman
	//private int salie = KLOOS_TYPE_KLOOS;
	
	// Messages for slideshow
	public static final int KLOOS_MSG_STOP = 0;
	public static final int KLOOS_MSG_RUNNING = 1;	
	public static final int KLOOS_ARG2_STOP = 0;
	public static final int KLOOS_ARG2_CONTINUE = 1729;
	// Delay for slideshow
	private static final int slideShowTime = 2500;

	// Constant for the Intent when inputing a new modulus
	private static final int CHOOSE_SUM = 2;

	private GestureDetector gesture;
	
	// Slideshow handler
	private Handler slideshow = new Handler(new Handler.Callback() {
		
		public boolean handleMessage(Message msg) {
			switch (msg.what) {	
			case KLOOS_MSG_STOP: // Stop slideshow -- currently unused!
				slideShow = false;
				return true;
			case KLOOS_MSG_RUNNING: // Slideshow is running
				if (msg.arg2 == KLOOS_ARG2_CONTINUE) {
					slideShow = true;
					Message nmsg = Message.obtain();
					nmsg.what = 1;
					// Is it really needed?
					nmsg.arg1 = kv.k.c+1;
					nmsg.arg2 = KLOOS_ARG2_CONTINUE;
					kv.createKloos(kv.k.m,kv.k.n,kv.k.c+1,onlyPrimes,kv.k.salie);
					boolean unused = slideshow.sendMessageDelayed(nmsg, slideShowTime);
					showModulusAndSum();
					kv.invalidate();
	    		// And continue...
				} else {		
					slideShow = false;
				}
				return true;
			}
		return false;
		}	
	});
	
	// Callback class for scroll and tap
	private class KloosGestureListener extends GestureDetector.SimpleOnGestureListener {

		float MINIMAL = 25.0f;

		@Override
		public boolean onSingleTapUp(MotionEvent event) {
			// Single press
			//Log.d("onSingleTapUp",event.toString());
			switch (event.getAction()) {
			case MotionEvent.ACTION_UP:
				if (slideShow) {
					Message msg = Message.obtain();
					msg.what = KLOOS_MSG_RUNNING;
					msg.arg1 = 2; // Should be modulus, but not used since we stop...
					// Stop the show
					msg.arg2 = KLOOS_ARG2_STOP;
					slideshow.removeMessages(1);
					slideshow.sendMessage(msg);
					// Indicate the show is stopped...
					Toast toast = Toast.makeText(kv.getContext(), "Slideshow stopped.",Toast.LENGTH_SHORT);
					toast.show();
					return true;
				} else {
					showModulusAndSum();
					return true;
				}
			}
			return true;
		}

		@Override
		public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX, float distanceY) {
			int c;

			//Log.d("onScroll",Float.toString(distanceX));
			//Log.d("onScroll",Float.toString(distanceY));
			if (distanceX>MINIMAL) {
				kv.createKloos(kv.k.m,kv.k.n,kv.k.c+1,onlyPrimes,kv.k.salie);
				kv.invalidate();
			}

			if (distanceX<-MINIMAL) {
				if (onlyPrimes) {
					c = previousPrime(kv.k.c-1);
				} else {
					c = kv.k.c-1;
				}
				// We pass false as onlyPrimes to avoid changing the modulus we described
				kv.createKloos(kv.k.m,kv.k.n,c,false,kv.k.salie);
				kv.invalidate();
			}

			return true;
		}

//		  @Override
//		  public void onShowPress(MotionEvent ev) {
//		    //Log.d("onShowPress",ev.toString());
//		  }
//		  @Override
//		  public void onLongPress(MotionEvent ev) {
//		    //Log.d("onLongPress",ev.toString());
//		  }
//		  @Override
//		  public boolean onDown(MotionEvent ev) {
//		    //Log.d("onDownd",ev.toString());
//		    return true;
//		  }
		  
//		  @Override
//		  public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) {
//		    //Log.d("Fling e1",e1.toString());
//		    //Log.d("Fling e2",e2.toString());
//		    //return true;
//		  }

	}
	       	
	
	// Utility to display modulus and sum quickly 
	
	public String infoSums() {
		
		String message = "";
		switch (kv.k.salie) {
		case KLOOS_TYPE_KLOOS:
			message = "Kloosterman sum\n";
			break;
		case KLOOS_TYPE_SALIE:
			message = "Salié sum\n";
			break;
		case KLOOS_TYPE_BOTH:
			break;
		}
		
		message = message + "Modulus = " +  Integer.toString(kv.k.c) + "; m= " + 
			Integer.toString(kv.k.m)+ "; n= " + Integer.toString(kv.k.n)+"\n";
		
		switch (kv.k.salie) {
		case KLOOS_TYPE_KLOOS:
			message = message + "Sum = " + kv.k.kr[kv.k.phic];
			break;
		case KLOOS_TYPE_SALIE:
			message  = message + "Sum = ";
			if (Math.abs(kv.k.sr[kv.k.phic])<0.00001) {
				if (Math.abs(kv.k.si[kv.k.phic])<0.00001) {
					message = message + "0";
				} else {
					message = message + kv.k.si[kv.k.phic] + " i";
				}
			} else {
				message = message + kv.k.sr[kv.k.phic];
			}
			break;
		case KLOOS_TYPE_BOTH:
			message = message + "K(red) = " + kv.k.kr[kv.k.phic] + "\nS(blue) = ";
			if (Math.abs(kv.k.sr[kv.k.phic])<0.00001) {
				if (Math.abs(kv.k.si[kv.k.phic])<0.00001) {
					message = message + "0";
				} else {
					message = message + kv.k.si[kv.k.phic] + " i";
				}
			} else {
				message = message + kv.k.sr[kv.k.phic];
			}
			break;
		} 
		return message;
	}
	
	public void showModulusAndSum () {
		
		String message = infoSums();
		Toast toast = Toast.makeText(kv.getContext(),message,Toast.LENGTH_SHORT);
		toast.show();
	}
	
	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		gesture = new GestureDetector(this, new KloosGestureListener());
		
		rng = new Random();
        kv = new KloosView(this);
        
        // Restore settings
        SharedPreferences settings = getSharedPreferences("Kloostermania", 0);
        int modulus = settings.getInt("modulus", 173);
        int m = settings.getInt("m",1);
        int n = settings.getInt("n",1);
        onlyPrimes = settings.getBoolean("primes", true);
        slideShow = settings.getBoolean("slide", false);
        int salie = KLOOS_TYPE_KLOOS;
        try {
        	// Older versions used a boolean setting for Salié sums
        	salie = settings.getInt("salie",KLOOS_TYPE_KLOOS);
        } catch (ClassCastException e) {
        	boolean s = settings.getBoolean("salie", false);
        	if (s) {
        		salie = KLOOS_TYPE_SALIE;
        	} else {
        		salie = KLOOS_TYPE_KLOOS;
        	}
        }
               
        // Create the view and display
        kv.createKloos(m,n,modulus,onlyPrimes,salie);
        //kv.setOnTouchListener(touch);
        
        // Don't start earlier since the data is not there yet otherwise...
        if (slideShow) {
    		startSlideShow();
        }
        setContentView(kv);
    }
	
	@Override
	public boolean onTouchEvent(MotionEvent event) {
	  if (gesture.onTouchEvent(event))
	    return true;
	  else
	    return false;
	}

	public void startSlideShow() {

		// Clear up the queue, if any
		slideshow.removeMessages(1);
		Message msg = Message.obtain();
		msg.what = KLOOS_MSG_RUNNING;
		msg.arg1 = kv.k.c;
		msg.arg2 = KLOOS_ARG2_CONTINUE;
		// Start sliding in 2.5 seconds
		showModulusAndSum();
		boolean unused = slideshow.sendMessageDelayed (msg, slideShowTime);

	}
	 
    // onPause is where the orientation change works...
	@Override
    protected void onPause(){
       super.onPause();
    
      // Stop the slideshow if running...
      if (slideShow) {
    	  slideshow.removeMessages(1);
      }
      // Save the settings
      SharedPreferences settings = getSharedPreferences("Kloostermania", 0);
      SharedPreferences.Editor editor = settings.edit();
      editor.putInt("modulus", kv.k.c);
      editor.putBoolean("primes", onlyPrimes);
      editor.putBoolean("slide", slideShow);
      editor.putInt("salie", kv.k.salie);
      editor.putInt("m", kv.k.m);
      editor.putInt("n", kv.k.n);
      // Commit must not be forgotten
      editor.commit();
    }

	
	// Listen for results.
	protected void onActivityResult(int requestCode, int resultCode, Intent data){	    

		// Get the modulus	
		int modulus = data.getIntExtra("net.frogparrot.android.kloostermania.modulus", 179);
		int m = data.getIntExtra("net.frogparrot.android.kloostermania.m",1);
		int n = data.getIntExtra("net.frogparrot.android.kloostermania.n",1);
		int salie = data.getIntExtra("net.frogparrot.android.kloostermania.type", KLOOS_TYPE_KLOOS); 
		kv.createKloos(m,n,modulus,onlyPrimes,salie);
		kv.invalidate();
	}

    /* Creates the menu items; onPrepare... is called every time the menu is opened, so
     * it is better for the dynamic toggle of the moduli... */
	
    public boolean onPrepareOptionsMenu(Menu menu) {
    	
    	menu.clear();

        menu.add(0, MENU_NEW_KLOOS, 0, "New sum");
        // No need to have next anymore in the menu
       // menu.add(0, MENU_NEXT,0, "Next");
        
        // Group 1 will help disable the menu during slideshow
        menu.add(1, MENU_SLIDE, 0, "Slideshow");
        menu.add(0, MENU_RANDOM, 0, "Random");
        if (!onlyPrimes) {
        	menu.add(0, MENU_PRIMES, 0, "Primes only" );
        } else { 
        	menu.add(0,MENU_PRIMES, 0, "All moduli" );
        }        
        menu.add(0, MENU_MOD, 0, "Modulus and sum");
        menu.add(0, MENU_ABOUT, 0, "About");
        // Disable during slideshow
        if (slideShow) { menu.setGroupEnabled(1,false); }
        return true;
    }
        
    
    /* Handles item selections */
    public boolean onOptionsItemSelected(MenuItem item) {
    	int modulus = 17;
    	
    	switch (item.getItemId()) {
    	case MENU_NEW_KLOOS:
    		Intent intent = new Intent(this, net.frogparrot.android.Choosesum.class);
    		intent.putExtra("net.frogparrot.android.kloostermania.modulus", kv.k.c);
    		intent.putExtra("net.frogparrot.android.kloostermania.type",kv.k.salie);
    		intent.putExtra("net.frogparrot.android.kloostermania.m",kv.k.m);
    		intent.putExtra("net.frogparrot.android.kloostermania.n",kv.k.n);
    		startActivityForResult(intent, CHOOSE_SUM);
    		return true;
//    	case MENU_NEXT:
//    		kv.createKloos(kv.k.m,kv.k.n,kv.k.c+1,onlyPrimes,kv.k.salie);
//    		kv.invalidate();
//    		return true;       
    	case MENU_SLIDE:
    		slideShow = true;
    		startSlideShow();
    		return true;
    	case MENU_RANDOM:
    		// Biased dice to select smaller moduli most of the time...
    		int choice = rng.nextInt(10);
    		switch (choice) {
    		case 0:	
    			modulus = 10000+rng.nextInt(10000);
    			break;
    		case 1:
    		case 2:
    		case 3:
    			modulus = 5000 + rng.nextInt(5000);
    			break;
    		case 4:
    		case 5:
    		case 6:
    			modulus = 1000+rng.nextInt(4000);
    		case 7:
    		case 8:
    		case 9:
    			modulus = 2+rng.nextInt(1000);
    		}
    		kv.createKloos(kv.k.m,kv.k.n,modulus,onlyPrimes,kv.k.salie);
    		showModulusAndSum();
    		kv.invalidate();        	
    		return true;
    	case MENU_PRIMES:
    		onlyPrimes = !onlyPrimes;
    		return true;
    	case MENU_MOD:
    		new AlertDialog.Builder(this)
    		.setMessage(infoSums())
    		.setPositiveButton("OK", new DialogInterface.OnClickListener() {
    			public void onClick(DialogInterface dialog) {
    			}
    			public void onClick(DialogInterface dialog, int which) {			
    			}})
    			.setOnCancelListener(new DialogInterface.OnCancelListener() {
    				public void onCancel(DialogInterface dialog) {               
    				}})
    				.show();
    		return true;
    	case MENU_ABOUT:
    		new AlertDialog.Builder(this)
    		.setMessage(Html.fromHtml("<b>Pocket Kloostermania</b><br>Version "+version+"<br>License: GPLv2<br><i>blogs.ethz.ch/kowalski/the-kloostermania-page</i>"))
    		.setPositiveButton("OK", new DialogInterface.OnClickListener() {
    			public void onClick(DialogInterface dialog) {
    			}
    			public void onClick(DialogInterface dialog, int which) {						
    			}})
    			.setOnCancelListener(new DialogInterface.OnCancelListener() {
    				public void onCancel(DialogInterface dialog) {               
    				}})
    				.show();
    		return true;

    	} // Switch
        	
        return false;
    }
    
    
    // Mathematical utilities:
    // * egcd
    // * naive primality test
    // * compute next prime
    
    public static int[] egcd(int p, int q) {
        if (q == 0)
           return new int[] { p, 1, 0 };

        int[] vals = egcd(q, p % q);
        int d = vals[0];
        int a = vals[2];
        int b = vals[1] - (p / q) * vals[2];
        return new int[] { d, a, b };
     }
    
    public boolean isPrime(int p) {
    	int j, k;
    	
    	if (p==2) { return true; }
    	if (p%2==0) { return false; }
    	k = (int) Math.sqrt(p)+1;
    	for(j=3;j<=k;j+=2) {
    		if (p%j==0) { return false; }
    	}
    	return true;
    }

    public int nextPrime(int p) {
    	/* Finds prime closest to m */
    	int n=p;
    	if (n<0) { n=2; }
    	if (n<=2) { return 2; }
    	while (!isPrime(n)) {
    		n=n+1;
    	}
    	return n;
    }
    
    public int previousPrime(int p) {
    	/* Finds prime closest to m */
    	int n=p;
    	if (n<0) { n=2; }
    	if (n<=3) { return 2; }
    	while (!isPrime(n)) {
    		n=n-1;
    	}
    	return n;
    }
    
    // Not used anymore at the moment
//	private int nextCoprime(int i, int c) {
//		int j = i;
//		while (egcd(j,c)[0] != 1) {
//		  j++;	
//		}
//		return j;
//	}

    class KloosData {
    	// We keep the Euler number as element of the data
    	// The arrays of real/imaginary parts are of size c
    	// and the elements beyond phic should not be accessed...
    	Double[] kr, ki, sr, si; // Kloosterman real and imaginary parts, and same for Salié
    	int[] legendres;
    	int m,n,c,phic;
    	int salie;
    	double[] bds;
    	
    	// The salié sum T(1,1;p) is not very fun because X^2-4 is quite split over the rationals...
		// So we use the convention that m, n gives T(m,-n;p) instead for Salié sums.
		
    	public KloosData(int m, int n, int c, int s) {
    		double xmin,xmax,ymin,ymax;
    		int nbcoprime;
    		
    		this.salie = s;
    		if (salie != KLOOS_TYPE_KLOOS) {
    			this.legendres = new int[c];
    			for(int i=1; i<c; i++) {
    				legendres[i]=-1;
    			}
    			for (int i=1; 2*i<c; i++) {
    				legendres[(i*i) % c]=1;
    			}
    		}
    		//Log.v("KL",c + " Create");
    		this.m = m;
    		this.n = n;
    		this.c = c;
    		this.phic = c-1;
    		
    		this.kr=new Double[c];
    		this.ki=new Double[c];
    		kr[0]=0.0;
        	ki[0]=0.0;
    		if (salie != KLOOS_TYPE_KLOOS) {
    			this.sr=new Double[c];
    			this.si=new Double[c];
    			sr[0]=0.0;
    			si[0]=0.0;
    		}
    		
    		int[] j;
    		xmin = -0.1;
    		xmax = 0.1;
    		ymin = -0.1;
    		ymax = 0.1;
        	
        	
        	nbcoprime = 0;
        	double dx = 1.0, dy = 1.0;
        	
        	if (salie != KLOOS_TYPE_KLOOS) {
        		for(int i=1; i<c; i++) {
        			j = egcd(i,c);
        			if (j[0] == 1) {
        				nbcoprime=nbcoprime+1;
        				sr[nbcoprime] = (double) (sr[nbcoprime-1]+legendres[i]*Math.cos(2*Math.PI/c*(m*i-n*j[1])));      
        				if (sr[nbcoprime]<xmin) { xmin = sr[nbcoprime]; }
        				if (sr[nbcoprime]>xmax) { xmax = sr[nbcoprime]; }

        				si[nbcoprime] = (double) (si[nbcoprime-1]+legendres[i]*Math.sin(2*Math.PI/c*(m*i-n*j[1])));
        				if (si[nbcoprime]<ymin) { ymin = si[nbcoprime]; }
        				if (si[nbcoprime]>ymax) { ymax = si[nbcoprime]; }     
        			}
        		}
        		dx = xmax-xmin;
        		dy = ymax-ymin;
        		this.phic = nbcoprime;
        	} 
        	
        	// Start again if needed
        	if (salie != KLOOS_TYPE_SALIE) {
        		nbcoprime = 0;
        		for(int i=1; i<c; i++) {
        			j = egcd(i,c);
        			if (j[0] == 1) {
        				nbcoprime=nbcoprime+1;
        				kr[nbcoprime] = (double) (kr[nbcoprime-1]+Math.cos(2*Math.PI/c*(m*i+n*j[1])));      
        				if (kr[nbcoprime]<xmin) { xmin = kr[nbcoprime]; }
        				if (kr[nbcoprime]>xmax) { xmax = kr[nbcoprime]; }

        				ki[nbcoprime] = (double) (ki[nbcoprime-1]+Math.sin(2*Math.PI/c*(m*i+n*j[1])));
        				if (ki[nbcoprime]<ymin) { ymin = ki[nbcoprime]; }
        				if (ki[nbcoprime]>ymax) { ymax = ki[nbcoprime]; }     
        			}
        		}
        		dx = xmax-xmin;
        		dy = ymax-ymin;
        		this.phic = nbcoprime;
        	}
        	
        	// Expand a bit the size on both sides for cleaner display
        	bds = new double[] {xmin-0.1*dx, xmax+0.1*dx, ymin-0.15*dy, ymax+0.15*dy};
    	}
    	
    	// Constructor without Salié sum, just copy the Kloosterman case
    	
    	public KloosData(int m, int n, int c) {
    		double xmin,xmax,ymin,ymax;
    		int nbcoprime;
    		
    		salie = KLOOS_TYPE_KLOOS;
    		//Log.v("KL",c + " Create");
    		this.m = m;
    		this.n = n;
    		this.c = c;
    		this.phic = c-1;
    		
    		this.kr=new Double[c];
    		this.ki=new Double[c];
    		
    		int[] j;
    		xmin = -0.1;
    		xmax = 0.1;
    		ymin = -0.1;
    		ymax = 0.1;
        	kr[0]=0.0;
        	ki[0]=0.0;
        	
        	nbcoprime = 0;
        	for(int i=1; i<c; i++) {
        		j = egcd(i,c);
        		if (j[0] == 1) {
        			nbcoprime=nbcoprime+1;
        			kr[nbcoprime] = (double) (kr[nbcoprime-1]+Math.cos(2*Math.PI/c*(m*i+n*j[1])));      
        			if (kr[nbcoprime]<xmin) { xmin = kr[nbcoprime]; }
        			if (kr[nbcoprime]>xmax) { xmax = kr[nbcoprime]; }

        			ki[nbcoprime] = (double) (ki[nbcoprime-1]+Math.sin(2*Math.PI/c*(m*i+n*j[1])));
        			if (ki[nbcoprime]<ymin) { ymin = ki[nbcoprime]; }
        			if (ki[nbcoprime]>ymax) { ymax = ki[nbcoprime]; }     
        		}
        	}
        	double dx = xmax-xmin;
        	double dy = ymax-ymin;
        	this.phic = nbcoprime;
        	// Expand a bit the size on both sides for cleaner display
        	bds = new double[] {xmin-0.1*dx, xmax+0.1*dx, ymin-0.15*dy, ymax+0.15*dy};
    	}
    }
    
    class KloosView extends View {

		//Context ctx;
    	KloosData k;    
    	Paint axes;
    	Paint curvek;
    	Paint curves;
    	
		public KloosView(Context context) {
			super(context);

			axes = new Paint();
	        axes.setColor(Color.GRAY);
	        axes.setStrokeWidth(2);  
	        // Kloosterman is red
	        curvek = new Paint();
	        curvek.setColor(Color.RED);
	        curvek.setStrokeWidth(1);       
	        curvek.setAntiAlias(true);
	        curvek.setStrokeCap(Paint.Cap.ROUND);
	        // Salié is green
	        curves = new Paint();
	        curves.setColor(Color.BLUE);
	        curves.setStrokeWidth(1);       
	        curves.setAntiAlias(true);
	        curves.setStrokeCap(Paint.Cap.ROUND);
	     
	        setFocusable(true);

		}
		
		public void createKloos(int a, int b, int q, boolean onlyPrimes, int salie) {
			int m=q;
				
			//Log.v("KL",q+ " create kloos");
			if (onlyPrimes == true) { m = nextPrime(m); }
			k = new KloosData(a,b,m, salie); 
		}
		
		@Override
        public void onDraw(Canvas canvas) {
		
			//Log.v("KL","onDraw "+ k.c);
			canvas.drawColor(Color.WHITE);
			int h=canvas.getHeight();
			int w=canvas.getWidth();

			if (k != null) {
				// x and y pixel scales
				float xsc = (float) (k.bds[1]-k.bds[0])/w;
				float ysc = (float) (k.bds[3]-k.bds[2])/h;
				
				float cx = (float) -k.bds[0]/xsc;
				float cy = (float) ((float) h+k.bds[2]/ysc);
				
				canvas.drawLine(cx,0,cx,h, axes);		
				canvas.drawLine(0,cy-25,w,cy-25, axes);
				
				// Loop over the part of the arrays that actually exists...
				if (k.salie != KLOOS_TYPE_SALIE) {
					for (int i=0; i<k.phic ; i++) {

						float sx = cx+k.kr[i].floatValue()/xsc;
						float sy = cy-k.ki[i].floatValue()/ysc;
						float ex = cx+k.kr[i+1].floatValue()/xsc;
						float ey = cy-k.ki[i+1].floatValue()/ysc;
						canvas.drawLine(sx,sy-25,ex,ey-25,curvek);

					}
				}
				// Same for Salié
				if (k.salie != KLOOS_TYPE_KLOOS) {
					for (int i=0; i<k.phic ; i++) {

						float sx = cx+k.sr[i].floatValue()/xsc;
						float sy = cy-k.si[i].floatValue()/ysc;
						float ex = cx+k.sr[i+1].floatValue()/xsc;
						float ey = cy-k.si[i+1].floatValue()/ysc;
						canvas.drawLine(sx,sy-25,ex,ey-25,curves);

					}
				}

			}
			invalidate();
		}


    }
}
	
